/*
    a dog dimmer

    v0.01 - initial

    -detect channel
    -detect aspect, frame size
    -disable if in guide or some other menus
    -update no more then every 40ms ?
tbd:
    -make it work with remote_extender
    -detect if dog is present
    -check if anti dog is available
    -set the transparancy to 0%?

*/
#include <string.h>
#include <tap.h>
#include "exTap.h"
#include "emma.h"
#include "tf5800.h"
#include "dog_dimmer.h"
#include "bitmap.h"

#ifdef _WIN32
#else
#define ID_USERTAP 0xbdb02001
TAP_ID(ID_USERTAP);

TAP_PROGRAM_NAME("dog_dimmer v0.01");
TAP_AUTHOR_NAME("bdb");
TAP_DESCRIPTION("a dog dimmer v0.01");
TAP_ETCINFO(__DATE__);
#endif

//----------------------------------------------------------------------------
#define MAX_CHANNELS 100;

//in dogs.h
//type_bitmap *bitmap_list[MAX_CHANNELS];
#include "dogs.h"
int num_channels = sizeof(bitmap_list) / sizeof(type_bitmap);

word *bitmap_1555 = NULL;
word *bitmap_mask_1555 = NULL;
byte *bitmap_dog8 = NULL;
//byte *bitmap_dog16 = NULL;
//byte *bitmap_dog24 = NULL;
word *bitmap_magnified_1555 = NULL;
byte *bitmap_r = NULL;
byte *bitmap_g = NULL;
byte *bitmap_b = NULL;
//----------------------------------------------------------------------------
int dog_dimming_enabled = 1;
int dog_detected = 0;
int dog_mode = NORMAL_DOG;
int magnified_dog = 0;

int two_seconds_enabled = 1;
dword two_seconds = 0;
int debug = 3;

word rgn = 0;
type_bitmap *channel;
int mode = 1;

//----------------------------------------------------------------------------
//
//tsr_commander stuff
//
#ifdef _WIN32
#else
#include "TSRCommander.inc"

bool TSRCommanderExitTAP()
{
    close();
    return FALSE;
}

void TSRCommanderConfigDialog()
{
    ShowMenu(&main_menu);
}

int is_tsr_commander_running(void)
{
    int i;
    type_TAPProcess *TAPProcess;

    for (i=0; i<16; i++) {
        TAPProcess = (type_TAPProcess *)(fw_tap_table + (i*32));
        if (TAPProcess->firmwaregp) {
            if (TAPProcess->header) {
                if (TAPProcess->header->id == 0x807a10b2) {  //tsr_commander
                    return 1;
                }
                if (TAPProcess->header->id == 0x81001021) {  //tap_commander
                    return 1;
                }
            }
        }
    }
    return 0;
}
#endif

int is_remote_extender_running(void)
{
    int i;
    type_TAPProcess *TAPProcess;

    for (i=0; i<16; i++) {
        TAPProcess = (type_TAPProcess *)(fw_tap_table + (i*32));
        if (TAPProcess->firmwaregp) {
            if (TAPProcess->header) {
                if (TAPProcess->header->id == 0x814243a3) {  //remote_extender
                    return 1;
                }
            }
        }
    }
    return 0;
}

//----------------------------------------------------------------------------
//
//main
//
char *tap_name;
type_menu main_menu = {
    0,
    0,
    {0},
    0,
    (char *)__tap_program_name__,
    {
        {7, "xxx",              },
        {0,  "Disable DOG Dimming",},
        {1,  "Set Black DOG",},
        {2,  "Display Magnified Dog",},
        {3,  "Show Current Service Id",},
        {4,  "Enable Serial Port Debug",},
        {5,  "Enable OSD Status",},

        {6,  "Exit TAP",},
    },
};

//-----------------------------------------------------------------------------
int TAP_Main(void)
{
    int ret;

    tap_name =  (char *)__tap_program_name__;
    sprintf(strLog, "Opening %s", tap_name);
    log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_SHORT);

    //determine support for different firmwares
    ret = exTAP_Init();
    if (ret) {
        sprintf(strLog, "Unsupported platform - %d:%04x", fw_sysid, _appl_version);
        log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_MEDIUM);
        return 0;
    }

    //only if RE not running
    if (is_remote_extender_running() == 0) {
        ret = exTAP_Key_Extend2(1);
        if (ret < 0) TAP_Print("\nexTAP_Key_Extend2() failed, err=%d", ret);
    }
    main_menu.visible = 0;

    if (dog_dimming_enabled) {
        strcpy(main_menu.item[1].str, "Disable DOG Dimming");
    }
    else {
        strcpy(main_menu.item[1].str, "Enable DOG Dimming");
    }

    if (dog_mode == NORMAL_DOG) {
        strcpy(main_menu.item[2].str, "Set Horizontal DOG Mode");
    }
    else
    if (dog_mode == HORIZONTAL_DOG) {
        strcpy(main_menu.item[2].str, "Set Black DOG Mode");
    }
    else
    if (dog_mode == BLACK_DOG) {
        strcpy(main_menu.item[2].str, "Set Masked DOG Mode");
    }
    else
    if (dog_mode == MASKED_DOG) {
        strcpy(main_menu.item[2].str, "Set Black Masked DOG Mode");
    }
    else {
        strcpy(main_menu.item[2].str, "Set Normal DOG Mode");
    }

    if (debug & 2) {
        strcpy(main_menu.item[5].str, "Disable Serial Port Debug");
    }
    else {
        strcpy(main_menu.item[5].str, "Enable Serial Port Debug");
    }

    if (debug & 1) {
        strcpy(main_menu.item[6].str, "Disable OSD Status");
    }
    else {
        strcpy(main_menu.item[6].str, "Enable OSD Status");
    }

    channel = 0;
    init2();
    TSRCommanderInit(0, TRUE);
    return 1;                 //0=application mode, 1=tsr mode
}

int close(void)
{
    if (rgn) TAP_Osd_Delete(rgn);
    DeleteMenu(&main_menu);
    close2();

    sprintf(strLog, "Closing  %s", tap_name);
    log_message(LOG_SERIAL|LOG_SCREEN_WAIT, strLog, DELAY_SHORT);
    if (message_rgn) TAP_Osd_Delete(message_rgn);
    exTAP_Key_Extend2(0);
    if (OpLog) TAP_Hdd_Fclose(OpLog);
    TSRCommanderErase();
    TAP_Print("\nClosing %s", tap_name);
    TAP_Exit();
    return 0;
}

dword TAP_EventHandler(word event, dword param1, dword param2)
{
    dword state, subState;
    int ret;
    int tick;
    static int last_tick = 0;
    static int last_dog_tick = 0;

    switch (event) {
        case EVT_IDLE:
//            TAP_Print("\nEVT_IDLE, event=%x, param1=%x, param2=%x", event, param1, param2);
            tick = TAP_GetTick();
            if (tick == last_tick)  {
//                last_tick = tick;
//                break;
            }

            //
            //message window
            //
            if (message_showing) {
                if (tick > message_showing) {
                    TAP_Osd_Delete(message_rgn);
                    message_rgn = 0;
                    message_showing = 0;
                }
            }


            //
            //aspect ratio monitor
            //
            ret = exTAP_update_aspect_ratio();
            if (ret) if (debug & 2) display_aspect_ratio();
            if (ret & 0xc) {  //main changed
                on_channel_change();
                tick = TAP_GetTick();
            }


            //
            //housekeeping
            //
            if (two_seconds_enabled) {
                if (tick > two_seconds) {
                    two_second_update();
                    tick = TAP_GetTick();
                    two_seconds = tick + (2*100);
                }
            }

            //
            //mask dog
            //
            TAP_GetState(&state, &subState);
            if ((tick - last_dog_tick) > 2) {
                if (state == STATE_Normal) {
                    if (dog_dimming_enabled && channel && dog_detected) {
                        dog_dim(channel);
                        tick = TAP_GetTick();
                        last_dog_tick = tick;
                    }
                    //else should remove mask
//                    else if (rgn) TAP_Osd_Delete(rgn);
                }
            }

            TSRCommanderWork();
            last_tick = tick;
            break;

        case EVT_KEY:
//            TAP_Print("\nEVT_KEY, event=%x, param1=%x, param2=%x", event, param1, param2);
            TAP_GetState(&state, &subState);
//            TAP_Print("\nstate = %s, subState = %s", get_str(state_str, state), get_str(substate_str, subState));
            if (main_menu.visible) {
                param1 = main_menu_handler(&main_menu, param1);
            }
            else
            if ((state == STATE_Normal) && (subState == SUBSTATE_Normal)) {
                param1 = key_handler(param1, param2);
            }
            break;

        default:
//            TAP_Print("\nEVT_???, event=%x, param1=%x, param2=%x", event, param1, param2);
            break;
    }
    return param1;
}

int main_menu_handler(type_menu *menu, dword param1)
{
    int selection;
    int channel_type;
    int channel_number;
	TYPE_TapChInfo channel_info;

    switch (param1) {
        case RKEY_Ok:
            selection = TAP_Win_GetSelection(&menu->win);
            DeleteMenu(menu);
            switch (selection) {
                case 0: //enable
                    dog_dimming_enabled ^= 1;
                    if (dog_dimming_enabled) {
                        strcpy(main_menu.item[1].str, "Disable DOG Dimming");
                    }
                    else {
                        strcpy(main_menu.item[1].str, "Enable DOG Dimming");
                    }
                    return 0;
                case 1: //dog mode
                    dog_mode += 1;
                    dog_mode %= NUMBER_DOG_MODES;
                    if (dog_mode == NORMAL_DOG) {
                        strcpy(main_menu.item[2].str, "Set Horizontal DOG Mode");
                    }
                    else
                    if (dog_mode == HORIZONTAL_DOG) {
                        strcpy(main_menu.item[2].str, "Set Black DOG Mode");
                    }
                    else
                    if (dog_mode == BLACK_DOG) {
                        strcpy(main_menu.item[2].str, "Set Masked DOG Mode");
                    }
                    else
                    if (dog_mode == MASKED_DOG) {
                        strcpy(main_menu.item[2].str, "Set Black Masked DOG Mode");
                    }
                    else {
                        strcpy(main_menu.item[2].str, "Set Normal DOG Mode");
                    }
                    return 0;
                case 2:  //magnified dog
                    magnified_dog ^= 1;
                    if (magnified_dog == 0) {
                        TAP_Osd_Delete(rgn);
                	    rgn = TAP_Osd_Create(0, 0, 720, 576, 0, FALSE);
                    }
                    else {
                        if (channel) detect_dog(channel);
                        else magnified_dog = 0;
                    }
                    return 0;
                case 3: //service id
                    TAP_Channel_GetCurrent(&channel_type, &channel_number);
                    TAP_Channel_GetInfo(channel_type, channel_number, &channel_info);
                    sprintf(strLog, "Service ID = %04x", channel_info.svcId);
                    log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_MEDIUM);
//                    oprintf("Service ID = %04x", channel_info.svcId);
                    return 0;

                case 4: //serial debug
                    debug ^= 2;
                    if (debug & 2) {
                        strcpy(menu->item[5].str, "Disable Serial Port Debug");
                    }
                    else {
                        strcpy(menu->item[5].str, "Enable Serial Port Debug");
                    }
                    return 0;
                case 5: //osd debug
                    debug ^= 1;
                    if (debug & 1) {
                        strcpy(menu->item[6].str, "Disable OSD Status");
                    }
                    else {
                        strcpy(menu->item[6].str, "Enable OSD Status");
                    }
                    return 0;
                case 6: //exit
                    close();
                    return 0;
            }
            break;
        case RKEY_Recall:
            DeleteMenu(menu);
            return param1; // cascades key to other TAP's
        case RKEY_Exit:
            DeleteMenu(menu);
            return 0; // exit without Cascading
        default:
            TAP_Win_Action(&menu->win, param1); // send all other key's to menu-Win
            return 0;
    }
    return param1;
}

int key_handler(dword param1, dword param2)
{
    switch(param1) {
        //only if tsrcommander is not running
        case RKEY_Recall:
            if (!is_tsr_commander_running()) {
                ShowMenu(&main_menu);
                return 0;
            }
            break;

        case RKEY_Exit:
            //clear magnified dog
            if (magnified_dog) {
                magnified_dog = 0;
                if (rgn) {
                    TAP_Osd_Delete(rgn);
                    rgn = TAP_Osd_Create(0, 0, 720, 576, 0, FALSE);
                }
                return 0;
            }
            break;

        case RKEY_Record:
            if (magnified_dog) {
                //save as 8 bit bitmap
                save_bitmap(1, bitmap_dog8, channel->getpos.width, channel->getpos.height);
//                //save as 16 bit bitmap
//                save_bitmap(2, bitmap_dog16, channel->getpos.width, channel->getpos.height);
//                //save as 24 bit bitmap
//                save_bitmap(3, bitmap_dog24, channel->getpos.width, channel->getpos.height);
                return 0;
            }
            break;

        case RKEY_Info:
            if (magnified_dog) {
                sprintf(strLog, "Service ID = %04x, Aspect Ratio = %s\nx=%d, y=%d, width=%d, height=%d",
                    channel->service_id,
                    (fw_aspect[CHANNEL_Main].ratio==ASPECT_16_9) ? "16:9" :
                    (fw_aspect[CHANNEL_Main].ratio==ASPECT_4_3)  ? "4:3" : "???",
                    channel->getpos.x, channel->getpos.y, channel->getpos.width, channel->getpos.height
                );
                log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_MEDIUM);
                magnified_dog = 0;
                return 0;
            }
            break;

        case RKEY_Ok:
            if (magnified_dog) {
//                TAP_Print("\nRKEY_Ok");
                if (debug & 2) TAP_Print("\nget-(%d,%d), (%d,%d)", channel->getpos.x, channel->getpos.y, channel->getpos.width, channel->getpos.height);
                detect_dog(channel);
                return 0;
            }
            break;


        case RKEY_VolUp:
            if (param2 == RKEY_Right) {
                if (magnified_dog) {
//                    TAP_Print("\nRKEY_Right");
                    if (mode == 1) {
                        channel->getpos.x++;
                    }
                    if (mode == 2) {
                        channel->getpos.width++;
                    }
                    if (debug & 2) TAP_Print("\nget-(%d,%d), (%d,%d)", channel->getpos.x, channel->getpos.y, channel->getpos.width, channel->getpos.height);
                    detect_dog(channel);
                    return 0;
                }
            }
            break;
        case RKEY_VolDown:
            if (param2 == RKEY_Left) {
                if (magnified_dog) {
//                    TAP_Print("\nRKEY_Left");
                    if (mode == 1) {
                        channel->getpos.x--;
                    }
                    if (mode == 2) {
                        channel->getpos.width--;
                    }
                    if (debug & 2) TAP_Print("\nget-(%d,%d), (%d,%d)", channel->getpos.x, channel->getpos.y, channel->getpos.width, channel->getpos.height);
                    detect_dog(channel);
                    return 0;
                }
            }
            break;
        case RKEY_ChDown:
            if (param2 == RKEY_Down) {
                if (magnified_dog) {
//                    TAP_Print("\nRKEY_Down");
                    if (mode == 1) {
                        channel->getpos.y++;
                    }
                    if (mode == 2) {
                        channel->getpos.height--;
                    }
                    if (debug & 2) TAP_Print("\nget-(%d,%d), (%d,%d)", channel->getpos.x, channel->getpos.y, channel->getpos.width, channel->getpos.height);
                    detect_dog(channel);
                    return 0;
                }
            }
            break;
        case RKEY_ChUp:
            if (param2 == RKEY_Up) {
                if (magnified_dog) {
//                    TAP_Print("\nRKEY_Up");
                    if (mode == 1) {
                        channel->getpos.y--;
                    }
                    if (mode == 2) {
                        channel->getpos.height++;
                    }
                    if (debug & 2) TAP_Print("\nget-(%d,%d), (%d,%d)", channel->getpos.x, channel->getpos.y, channel->getpos.width, channel->getpos.height);
                    detect_dog(channel);
                    return 0;
                }
            }
            break;

        case RKEY_Red:
            if (magnified_dog) {
                mode = 1;
                return 0;
            }
            break;
        case RKEY_Green:
            if (magnified_dog) {
                mode = 2;
                return 0;
            }
            break;
        case RKEY_Yellow:
            break;
        case RKEY_Blue:
            if (magnified_dog) {
                load_bitmap();
                return 0;
            }
            break;
    }
    return param1;
}

//-----------------------------------------------------------------------------
int display_aspect_ratio()
{
    int j;
    int main_sub;

    for (j=0; j<1; j++) {
        main_sub = (j==0) ? CHANNEL_Main : CHANNEL_Sub;

        TAP_Print("\n%s Channel", (main_sub == CHANNEL_Sub) ? " Sub" : "Main");
        TAP_Print("        ");
        TAP_Print("%dx%d, ", fw_aspect[main_sub].horizontal_size, fw_aspect[main_sub].vertical_size);
        if      (fw_aspect[main_sub].ratio==ASPECT_4_3) TAP_Print(" 4:3");
        else if (fw_aspect[main_sub].ratio==ASPECT_16_9) TAP_Print("16:9");
        else if (fw_aspect[main_sub].ratio==ASPECT_2_21_1) TAP_Print("2.21:1");
        else                                TAP_Print("??:??");
        TAP_Print(", AFD_%x, ", fw_aspect[main_sub].afd);
        if (fw_aspect[main_sub].ratio == 2) {
            switch(fw_aspect[main_sub].afd) {
                case AFD0: TAP_Print("4:3 full frame image                          \n"); break;
                case AFD1: TAP_Print("4:3 full frame image                          \n"); break;
                case AFD2: TAP_Print("16:9 letterbox image                          \n"); break;
                case AFD3: TAP_Print("14:9 letterbox image                          \n"); break;
                case AFD5: TAP_Print("4:3 full frame image, alternative 14:9 center \n"); break;
                case AFD6: TAP_Print("16:9 letterbox image, alternative 14:9 center \n"); break;
                case AFD7: TAP_Print("16:9 letterbox image, alternative 4:3 center  \n"); break;
                default  : TAP_Print("Reserved                                      \n"); break;
            }
        }
        else if (fw_aspect[main_sub].ratio == 3) {
            switch(fw_aspect[main_sub].afd) {
                case AFD0: TAP_Print("16:9 full frame image                         \n"); break;
                case AFD1: TAP_Print("4:3 pillarbox image                           \n"); break;
                case AFD2: TAP_Print("16:9 full frame image                         \n"); break;
                case AFD3: TAP_Print("14:9 pillarbox image                          \n"); break;
                case AFD5: TAP_Print("4:3 pillarbox image, alternative 14:9 center  \n"); break;
                case AFD6: TAP_Print("16:9 full frame image, alternative 14:9 center\n"); break;
                case AFD7: TAP_Print("16:9 full frame image, alternative 4:3 center \n"); break;
                default  : TAP_Print("Reserved                                      \n"); break;
            }
        }
    }
}

//-----------------------------------------------------------------------------
//
//a bit naff, really want a callback
//
void two_second_update()
{
    TYPE_PlayInfo pi;        //overflows!
    byte dummy2[0x80];
    static int last_playMode = PLAYMODE_None;
    int tvRadio;
    int chNum;
    static int last_tvRadio = -1;
    static int last_chNum = -1;

    //playmode changed
    TAP_Hdd_GetPlayInfo(&pi);
    if (pi.playMode != last_playMode) {
//        if (debug & 2) TAP_Print("\n%s to %s", get_str(playmode_str, last_playMode), get_str(playmode_str, pi.playMode));
        on_channel_change();
        last_playMode = pi.playMode;
    }

    //channel changed
    TAP_Channel_GetCurrent(&tvRadio, &chNum);
    if (!((tvRadio == last_tvRadio) && (chNum == last_chNum))) {
//        if (debug & 2) TAP_Print("\n%s:%d to %s:%d", (last_tvRadio==SVC_TYPE_Tv)?"tv":"radio", last_chNum, (tvRadio==SVC_TYPE_Tv)?"tv":"radio", chNum);
        on_channel_change();
        last_tvRadio = tvRadio;
        last_chNum = chNum;
    }

    //
    //detect dog
    //need to figure out whether the program is currently displaying a dog or not
    //try correlating a screen capture with the dog mask
    //
    dog_detected = 1;
}


//-----------------------------------------------------------------------------
int init2()
{
    //Dump the Y frame buffer into a linear buffer
    //The frame buffer is arranged as 23x18 superblocks  32x32 pixels per superblock
    bitmap_1555 = (word *)TAP_MemAlloc(736 * 576 * sizeof(word));
    bitmap_mask_1555 = (word *)TAP_MemAlloc(736 * 576 * sizeof(word));
    bitmap_dog8 = (byte *)TAP_MemAlloc(1 * 736 * 576 * sizeof(byte));
//    bitmap_dog16 = (byte *)TAP_MemAlloc(2 * 736 * 576 * sizeof(byte));
//    bitmap_dog24 = (byte *)TAP_MemAlloc(3 * 736 * 576 * sizeof(byte));
    bitmap_magnified_1555 = (word *)TAP_MemAlloc(736 * 576 * sizeof(word));
    bitmap_r = (byte *)TAP_MemAlloc(736 * 576 * sizeof(byte));
    bitmap_g = (byte *)TAP_MemAlloc(736 * 576 * sizeof(byte));
    bitmap_b = (byte *)TAP_MemAlloc(736 * 576 * sizeof(byte));
    return 0;
}

int close2()
{
    TAP_MemFree(bitmap_1555);
    TAP_MemFree(bitmap_mask_1555);
    TAP_MemFree(bitmap_dog8);
//    TAP_MemFree(bitmap_dog16);
//    TAP_MemFree(bitmap_dog24);
    TAP_MemFree(bitmap_magnified_1555);
    TAP_MemFree(bitmap_r);
    TAP_MemFree(bitmap_g);
    TAP_MemFree(bitmap_b);
    return 0;
}


//-----------------------------------------------------------------------------
//
//
//
int on_channel_change()
{
    TYPE_PlayInfo pi;        //overflows!
    byte dummy2[0x80];
    int channel_type;
    int channel_number;
	TYPE_TapChInfo channel_info;
	int i;

    //delete any old dog masks
    if (rgn) {
        TAP_Osd_Delete(rgn);
        rgn = TAP_Osd_Create(0, 0, 720, 576, 0, FALSE);
    }

    //check if playing file
    TAP_Hdd_GetPlayInfo(&pi);
    if (pi.playMode == PLAYMODE_None) {
        TAP_Channel_GetCurrent(&channel_type, &channel_number);
    }
    else {
        //get channel from .rec header
        channel_type = pi.svcType;
        channel_number = pi.svcNum;
    }
    TAP_Channel_GetInfo(channel_type, channel_number, &channel_info);

//    if (channel_type == SVC_TYPE_Radio) return 0;

    channel = 0;
    for (i=0; i<num_channels; i++) {
        if ((bitmap_list[i].service_id == channel_info.svcId) &&
            (((bitmap_list[i].aspect_ratio == 169) && (fw_aspect[CHANNEL_Main].ratio==ASPECT_16_9)) ||
             ((bitmap_list[i].aspect_ratio == 43)  && (fw_aspect[CHANNEL_Main].ratio==ASPECT_4_3)))) {
                channel = &bitmap_list[i];
                break;
        }
    }
    if (channel == 0) {
        if (debug & 1) {
            sprintf(strLog, "DOG Mask not found: Service ID = %04x, Aspect Ratio = %s", channel_info.svcId,
                (fw_aspect[CHANNEL_Main].ratio==ASPECT_16_9) ? "16:9" :
                (fw_aspect[CHANNEL_Main].ratio==ASPECT_4_3)  ? "4:3" : "???"
            );
            log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_MEDIUM);
//            oprintf("DOG Mask not found: Service ID = %04x, Aspect Ratio = %s", channel_info.svcId,
//                (fw_aspect[CHANNEL_Main].ratio==ASPECT_16_9) ? "16:9" :
//                (fw_aspect[CHANNEL_Main].ratio==ASPECT_4_3)  ? "4:3" : "???"
//            );
        }
    }

    return 0;
}

//-----------------------------------------------------------------------------
//read dog from screen buffer
//magnify + display on OSD
//
int detect_dog(type_bitmap *channel)
{
    byte *picAddr;
    byte *uvAddr;
    dword x,y;
    dword xx,yy;
    byte r,g,b;
    int xpos;
    int ypos,ypos2;
    int yPos, cPos;
    word background;
    int magnify, magnify_w, magnify_h;
    int i,j;
    word value1555;

    magnified_dog = 1;
    //Retrieve the address of the first Y frame buffer from the memory controller
    picAddr = (byte *)(((*(dword *)EMMA_WMIF_ATBL0_Y & 0xffff0000) >> 6) | 0xa0000000);
    uvAddr  = (byte *)(((*(dword *)EMMA_WMIF_ATBL0_C & 0xffff0000) >> 6) | 0xa0000000);


    //scale the display
    magnify_w = (720-100) / channel->getpos.width;
    magnify_h = (576-100) / channel->getpos.height;
    magnify = (magnify_w > magnify_h) ? magnify_h : magnify_w;      //use the smaller

    //get the dog
	for (y=0; y<channel->getpos.height; y++) {
 		yy  = channel->getpos.y+y;
 		ypos  = ((yy & 0xffffffe0) * 736) + (yy&0x1f)*32;
		ypos2 = (((yy/2) & 0xffffffe0) * 736)  + ((yy/2)&0x1f)*32;
		for (x=0; x<channel->getpos.width; x++) {
     		xx  = channel->getpos.x+x;
			xpos =  ((xx & 0xffffffe0)<<5) + (xx&0x1f);
			yPos  = ypos  + xpos;
			cPos = (ypos2 + xpos)&0xfffffffe;

			YUV2RGB(picAddr[yPos], uvAddr[cPos+1], uvAddr[cPos], &r, &g, &b);

            //greyscale
            bitmap_dog8[(channel->getpos.height-1-y)*channel->getpos.width + x + 0] = picAddr[yPos];

//            bitmap_dog24[(channel->getpos.height-1-y)*3*channel->getpos.width + 3*x + 0] = r;
//            bitmap_dog24[(channel->getpos.height-1-y)*3*channel->getpos.width + 3*x + 1] = g;
//            bitmap_dog24[(channel->getpos.height-1-y)*3*channel->getpos.width + 3*x + 2] = b;

            r = (r>>3)&0x1f;
            g = (g>>3)&0x1f;
            b = (b>>3)&0x1f;
            value1555 = ((0x1<<15) | (r<<10) | (g<<5) | (b<<0));

//            //BMP is little endian
//            bitmap_dog16[(channel->getpos.height-1-y)*2*channel->getpos.width + 2*x + 0] = (value1555 >> 0) & 0xff;
//            bitmap_dog16[(channel->getpos.height-1-y)*2*channel->getpos.width + 2*x + 1] = (value1555 >> 8) & 0xff;

            for (j=0; j<magnify; j++) {
                for (i=0; i<magnify; i++) {
                    bitmap_magnified_1555[(magnify*y+j)*(magnify*channel->getpos.width) + (magnify*x+i)] = value1555;
                }
            }
		}
	}

    if (rgn) {
        TAP_Osd_Delete(rgn);
        rgn = 0;
    }

    //make sure the osd window is initialised
	if (rgn == 0) rgn = TAP_Osd_Create(0, 0, 720, 576, 0, FALSE);

    //centre + display magnified dog
    TAP_Osd_DrawPixmap(rgn, (720-1-magnify*channel->getpos.width)/2, (576-1-magnify*channel->getpos.height)/2, magnify*channel->getpos.width, magnify*channel->getpos.height, &bitmap_magnified_1555[0], TRUE, OSD_1555);

    return 0;
}

//-----------------------------------------------------------------------------
//read colour behind dog
//draw inverse dog in background colour
//
int dog_dim(type_bitmap *channel)
{
    byte *picAddr;
    byte *uvAddr;
    dword x,y;
    int i;
    byte r,g,b;
    int xpos;
    int ypos,ypos2;
    int yPos, cPos;
    word background;
    int top_r,top_g,top_b;
    int bot_r,bot_g,bot_b;
    int xx,yy;
    int setpos_x, setpos_y, setpos_width, setpos_height;
    word value1555;

    //Retrieve the address of the first Y frame buffer from the memory controller
    picAddr = (byte *)(((*(dword *)EMMA_WMIF_ATBL0_Y & 0xffff0000) >> 6) | 0xa0000000);
    uvAddr  = (byte *)(((*(dword *)EMMA_WMIF_ATBL0_C & 0xffff0000) >> 6) | 0xa0000000);


    //get the dog
	for (y=0; y<channel->getpos.height; y++) {
 		yy  = channel->getpos.y+y;
 		ypos  = ((yy & 0xffffffe0) * 736) + (yy&0x1f)*32;
		ypos2 = (((yy/2) & 0xffffffe0) * 736)  + ((yy/2)&0x1f)*32;
		for (x=0; x<channel->getpos.width; x++) {
     		xx  = channel->getpos.x+x;
			xpos =  ((xx & 0xffffffe0)<<5) + (xx&0x1f);
			yPos  = ypos  + xpos;
			cPos = (ypos2 + xpos)&0xfffffffe;

			YUV2RGB(picAddr[yPos], uvAddr[cPos+1], uvAddr[cPos], &r, &g, &b);
            bitmap_r[y*channel->getpos.width + x] = r;
            bitmap_g[y*channel->getpos.width + x] = g;
            bitmap_b[y*channel->getpos.width + x] = b;
		}
	}


    setpos_x      = (channel->getpos.x      * 720) / fw_aspect[CHANNEL_Main].horizontal_size;
    setpos_y      = (channel->getpos.y      * 576) / fw_aspect[CHANNEL_Main].vertical_size;
    setpos_width  = (channel->getpos.width  * 720) / fw_aspect[CHANNEL_Main].horizontal_size;
    setpos_height = (channel->getpos.height * 576) / fw_aspect[CHANNEL_Main].vertical_size;

    if (dog_mode == HORIZONTAL_DOG) {
        for (x=0; x<setpos_width; x++) {
            //scale osd to frame buffer, osd is always 720 wide, frame buffer may be 720,540, etc
            xx = (x * fw_aspect[CHANNEL_Main].horizontal_size) / 720;

            //
            //get the background along the top/bottom edges of the dog
            //
            top_r = bitmap_r[(0                       )*channel->getpos.width + xx];
            top_g = bitmap_g[(0                       )*channel->getpos.width + xx];
            top_b = bitmap_b[(0                       )*channel->getpos.width + xx];
            r = (top_r>>3)&0x1f;
            g = (top_g>>3)&0x1f;
            b = (top_b>>3)&0x1f;
            value1555 = ((0x1<<15) | (r<<10) | (g<<5) | (b<<0));
            for (y=0; y<setpos_height/2; y++) {
                bitmap_1555[y*setpos_width+x] = value1555;
            }

            bot_r = bitmap_r[(channel->getpos.height-1)*channel->getpos.width + xx];
            bot_g = bitmap_g[(channel->getpos.height-1)*channel->getpos.width + xx];
            bot_b = bitmap_b[(channel->getpos.height-1)*channel->getpos.width + xx];
            r = (bot_r>>3)&0x1f;
            g = (bot_g>>3)&0x1f;
            b = (bot_b>>3)&0x1f;
            value1555 = ((0x1<<15) | (r<<10) | (g<<5) | (b<<0));
            for (y=setpos_height/2; y<setpos_height; y++) {
                bitmap_1555[y*setpos_width+x] = value1555;
            }
        }
    }
    else
    //draw a black dog
    if (dog_mode == BLACK_DOG) {
        for (i=0; i<(setpos_width * setpos_height); i++) {
            bitmap_1555[i] = 0x8000;
        }
    }
    else
    if (dog_mode == MASKED_DOG) {
        for (x=0; x<setpos_width; x++) {
            //scale osd to frame buffer, osd is always 720 wide, frame buffer may be 720,540, etc
            xx = (x * fw_aspect[CHANNEL_Main].horizontal_size) / 720;

            //
            //get the background along the top/bottom edges of the dog
            //better: if mask set, use nearest neighbour, else transparent
            //
            top_r = bitmap_r[(0                       )*channel->getpos.width + xx];
            top_g = bitmap_g[(0                       )*channel->getpos.width + xx];
            top_b = bitmap_b[(0                       )*channel->getpos.width + xx];
            bot_r = bitmap_r[(channel->getpos.height-1)*channel->getpos.width + xx];
            bot_g = bitmap_g[(channel->getpos.height-1)*channel->getpos.width + xx];
            bot_b = bitmap_b[(channel->getpos.height-1)*channel->getpos.width + xx];

            for (y=0; y<setpos_height; y++) {
                yy = setpos_height - 1 - y;

                r = ((bot_r * y) + (top_r * (setpos_height-1-y)))/(setpos_height);
                g = ((bot_g * y) + (top_g * (setpos_height-1-y)))/(setpos_height);
                b = ((bot_b * y) + (top_b * (setpos_height-1-y)))/(setpos_height);
                r = (r>>3)&0x1f;
                g = (g>>3)&0x1f;
                b = (b>>3)&0x1f;
                value1555 = ((0x1<<15) | (r<<10) | (g<<5) | (b<<0));
                bitmap_1555[y*setpos_width+x] = (bitmap_mask_1555[setpos_width*y+x] & 0x8000) ? value1555 : bitmap_mask_1555[setpos_width*y+x];
            }
        }
    }
    else
    if (dog_mode == BLACK_MASKED_DOG) {
        for (x=0; x<setpos_width; x++) {
            //scale osd to frame buffer, osd is always 720 wide, frame buffer may be 720,540, etc
            xx = (x * fw_aspect[CHANNEL_Main].horizontal_size) / 720;

            //
            //if mask set, use black, else transparent
            //
            for (y=0; y<setpos_height; y++) {
                yy = setpos_height - 1 - y;

                bitmap_1555[y*setpos_width+x] = (bitmap_mask_1555[setpos_width*y+x] & 0x8000) ? 0x8000 : bitmap_mask_1555[setpos_width*y+x];
            }
        }
    }
    //interpolated dog
    else {
        for (x=0; x<setpos_width; x++) {
            //scale osd to frame buffer, osd is always 720 wide, frame buffer may be 720,540, etc
            xx = (x * fw_aspect[CHANNEL_Main].horizontal_size) / 720;

            //
            //get the background along the top/bottom edges of the dog
            //
            top_r = bitmap_r[(0                       )*channel->getpos.width + xx];
            top_g = bitmap_g[(0                       )*channel->getpos.width + xx];
            top_b = bitmap_b[(0                       )*channel->getpos.width + xx];
            bot_r = bitmap_r[(channel->getpos.height-1)*channel->getpos.width + xx];
            bot_g = bitmap_g[(channel->getpos.height-1)*channel->getpos.width + xx];
            bot_b = bitmap_b[(channel->getpos.height-1)*channel->getpos.width + xx];

            for (y=0; y<setpos_height; y++) {
                //if (channel->bitmap_1555[y*channel->getpos.width+x])
                r = ((bot_r * y) + (top_r * (setpos_height-1-y)))/(setpos_height);
                g = ((bot_g * y) + (top_g * (setpos_height-1-y)))/(setpos_height);
                b = ((bot_b * y) + (top_b * (setpos_height-1-y)))/(setpos_height);
                r = (r>>3)&0x1f;
                g = (g>>3)&0x1f;
                b = (b>>3)&0x1f;
                value1555 = ((0x1<<15) | (r<<10) | (g<<5) | (b<<0));
                bitmap_1555[y*setpos_width+x] = value1555;
            }
        }
    }


    //make sure the osd window is initialised
	if (rgn == 0) rgn = TAP_Osd_Create(0, 0, 720, 576, 0, FALSE);

    //draw dog
    TAP_Osd_DrawPixmap(rgn, setpos_x, setpos_y, setpos_width, setpos_height, bitmap_1555, TRUE, OSD_1555);


    return 0;
}



